@@ -39,6 +39,8 @@ static gint  imapx_url_equal (gconstpointer a, gconstpointer b);
 CamelProviderConfEntry imapx_conf_entries[] = {
 		{ CAMEL_PROVIDER_CONF_SECTION_START, "mailcheck", NULL,
 	  N_("Checking for New Mail") },
+	{ CAMEL_PROVIDER_CONF_CHECKBOX, "use_qresync", NULL,
+	  N_("Use _Quick Resync if the server supports it"), "1" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "use_idle", NULL,
 	  N_("Use I_dle if the server supports it"), "1" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "check_all", NULL,
@@ -53,10 +55,8 @@ CamelProviderConfEntry imapx_conf_entries[] = {
 	  N_("_Use custom command to connect to server"), "0" },
 	{ CAMEL_PROVIDER_CONF_ENTRY, "command", "use_command",
 	  N_("Command:"), "ssh -C -l %u %h exec /usr/sbin/dovecot --exec-mail imap" },
-#if 0
 	{ CAMEL_PROVIDER_CONF_CHECKSPIN, "cachedconn", NULL,
 	  N_("Numbe_r of cached connections to use"), "y:1:5:7" },
-#endif
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 #endif
 	{ CAMEL_PROVIDER_CONF_SECTION_START, "folders", NULL,
@@ -113,7 +113,6 @@ CamelServiceAuthType camel_imapx_password_authtype = {
 
 void camel_imapx_module_init(void);
 
-extern void camel_exception_setup(void);
 extern void imapx_utils_init(void);
 
 void
@@ -127,7 +126,6 @@ camel_imapx_module_init(void)
 	imapx_provider.translation_domain = GETTEXT_PACKAGE;
 
 	/* TEMPORARY */
-	camel_exception_setup();
 	imapx_utils_init();
 
 	camel_provider_register(&imapx_provider);
